/*  disasmTCC730, decode TCC730 (CalmRISC16 / MAC2424) machine code
         to assembly mnemonics.
    file: util.c, functions to interact with the system. 

    Copyright (C) 2004  Michael R. Donat <michael@donat.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdlib.h>
#include <sys/stat.h>
#include <stdio.h>
#include <fcntl.h>
#ifndef _BORLAND_
 #include <sys/ioctl.h>
 #include <unistd.h>
 #include <sys/io.h>
#else
 #include <io.h>
#endif
#include <string.h>
#include "types.h"
#include "util.h"

u32 lu32( u8* src ) {
   return (u32)((*(src + 0) <<  0 ) |
                (*(src + 1) <<  8 ) |
                (*(src + 2) << 16 ) |
                (*(src + 3) << 24 ));
}

u32 bu32( u8* src ) {
   return (u32)((*(src + 3) <<  0 ) |
                (*(src + 2) <<  8 ) |
                (*(src + 1) << 16 ) |
                (*(src + 0) << 24 ));
}

u16 lu16( u8* src ) {
   return (u16)((*(src + 0) <<  0 ) |
                (*(src + 1) <<  8 ));
}

u16 bu16( u8* src ) {
   return (u16)((*(src + 1) <<  0 ) |
                (*(src + 0) <<  8 ));
}

u32 chksum32( u8* src, u32 len ) {
   u32 sum = 0;
   u8* end = src + len;
   while( src < end )
      sum = ( sum + *(src++) );
   return sum;
}

int strrpos( char* str, const char c ) {
   int i = strlen( str );
   while( i > 0 && str[i] != c ) --i;
   return i;
}

char* cstr( char* dest, u8* src, u32 len ) {
   memcpy( dest, src, len );
   dest[len] = 0;
   return dest;
}

int loadFile( char* filename, u8 **buf ) {
   int aosfile;
   u32 length;
   u8* buffer;
   u32 bytesread = 0;
   struct stat filestats;
   if( !(aosfile = open( filename, O_RDONLY | O_BINARY ))) {
      fprintf( stderr, "Error opening input file: %s\n", filename );
      return -1;
   }
   if( fstat( aosfile, &filestats ) ) {
      fprintf( stderr, "Error accessing input file\n" );
      close( aosfile );
      return -1;
   }
   length = filestats.st_size;
   buffer = (u8*)malloc( length );
   while( bytesread < length )
      bytesread += read( aosfile, buffer + bytesread, length - bytesread );
   close( aosfile );
   *buf = buffer;
   return length;
}

int writeFile( char* filename, u8* buffer, u32 length ) {
   int binfile;
   struct stat filestats;
   if( !(binfile = open( filename, O_WRONLY | O_BINARY | O_CREAT | O_TRUNC,
                                   S_IWRITE | S_IREAD	))) {
      fprintf( stderr, "Error opening output file: %s\n", filename );
      return -1;
   }
   if( fstat( binfile, &filestats ) ) {
      fprintf( stderr, "Error accessing output file\n" );
      close( binfile );
      return -1;
   }
   if( !write( binfile, buffer, length ))
      fprintf( stderr, "Error writing to file\n" );
   close( binfile );
   return 0;
}

